# Copyright by Bernhard M. Wiedemann
# Licenced under GNU GPL
use strict;
#use warnings;
use xbase;
use ipcalc;

our %hostlistoptions;
do "hostlist.settings";
$hostlistoptions{infile}||="hostlist.in";


open(my $hostlist, $hostlistoptions{infile});
my @hostlist=<$hostlist>;
close($hostlist);

push(@::contextstarthooks, sub{ my($context)=@_;
   my $s=$::s{$context};
   $s->{date}=$^T;
   $s->{rdomain6}=ipcalc::ip6_to_rev($hostlistoptions{prefix6});
   $s->{domain}=$hostlistoptions{domain};


my %dupmap=(ippart=>{}, ether=>{}, name=>{});

foreach (@hostlist) {
   s/[ \t\r\n]+/ /g;
   s/#.*//; # strip comments
   my ($ippart,$ether,$name,$flags)=split(" ", $_);
   foreach my $x ($ether,$name) {$x=lc($x);} # canonical strings
   next unless $name;

# check dup
   for my $n (qw"ippart ether name") {
	my $map=$dupmap{$n};
	my $key=eval('$'.$n);
	if($map->{$key}) {
		die "$n collision of $ippart,$ether,$name with $map->{$key}";
	}
	$map->{$key}="$ippart,$ether,$name";
   }
   
   my $ip4=$hostlistoptions{subnet}.".".$ippart;
   my $rev4=join(".",reverse(split('\.', $ippart)));
   if($flags!~m/-4/) {
	   $s->{fdns}.="$name\tIN A $ip4\n$name\tIN MX 10 $hostlistoptions{mailhost}\n";
	   $s->{rdns}.="$rev4\tIN PTR $name.$hostlistoptions{domain}.\n";
   }
   $s->{hosts}.="$ip4\t$name.$hostlistoptions{domain}\t$name\n";

   $ether="\L$ether";
   $s->{ethers}.="$ip4 $ether\n";
   my $dhcpd="     host $name {
       hardware ethernet $ether;
       fixed-address $ip4;
     }\n";
   $s->{dhcp}.=$dhcpd;
# IPv6
   my @eui64=ipcalc::mac_to_eui64($ether);
   my $ip6=$hostlistoptions{prefix}.":".join(":", @eui64);
   if($flags=~m/6/) {
      $s->{fdns6}.="$name\tIN AAAA $ip6\n";
   }
# even ipv4-only hosts can have reverse ipv6 mappings without causing trouble
   my $n6=ipcalc::ip6_to_rev(join("",@eui64));
   $s->{rdns6}.="$n6\tIN PTR $name.$hostlistoptions{domain}.\n";
#print "hostlist.pm\n$s->{fdns6}";
}

});

# prefilterhooks check include mtime
push(@::prefilterhooks, sub{
      my($context, $infilename, $basefilename, $outfilename, $needpost, $needcheck)=@_;
      my $c=getFileContent($infilename);
      if($c=~m/##(rdns|rdns6|fdns|fdns6|dhcp)##/g) {
         my $touchfilename=$infilename.".$context.hostlist.touch";
         my $t=getmtime($touchfilename);
         if($t<getmtime($hostlistoptions{infile})) {# || $t<getmtime("conf.d/hostlist.pm"))) {
            $_[5]=1; 
            diag("include $hostlistoptions{infile} changed => rechecking $basefilename"); 
            touch($touchfilename);
         }
      }
   });

1;
