# Copyright by Bernhard M. Wiedemann
# Licenced under GNU GPL
use strict;
use warnings;
use xbase;
use ipcalc;

our %tunneloptions;
do "tunnelinput.settings";

open(my $hostlist, $tunneloptions{infile}) or last;
my @hostlist=<$hostlist>;
close($hostlist);

push(@::contextstarthooks, sub{ my($context)=@_;
   my $s=$::s{$context};
   $s->{date}=$^T;
   $s->{rdomain6}=ipcalc::ip6_to_rev($tunneloptions{prefix6});
   $s->{domain}=$tunneloptions{domain};


my %dupmap=(name=>{}, ip=>{});

foreach (@hostlist) {
   s/[ \t\r\n]+/ /g;
   s/#.*//; # strip comments
   my ($name, undef,undef,$ippart,$flags)=split(/\s+/, $_);
   foreach my $x ($name) {$x=lc($x);} # canonical strings
   next unless $name;

# check dup
   for my $n (qw"ippart name") {
	my $map=$dupmap{$n};
	my $key=eval('$'.$n);
	if($map->{$key}) {
		die "$n collision of $ippart,$name with $map->{$key}";
	}
	$map->{$key}="$ippart,$name";
   }
   
   my $ip4=$tunneloptions{subnet}.".".$ippart;
   my $rev4=join(".",reverse(split('\.', $ippart)));
   if($flags!~m/-4/) {
	   $s->{fdns}.="$name\tIN A $ip4\n$name\tIN MX 10 $tunneloptions{mailhost}\n";
	   $s->{fdns}.="*.$name\tIN A $ip4\n*.$name\tIN MX 10 $tunneloptions{mailhost}\n";
	   $s->{fdns}.="${name}4\tIN A $ip4\n${name}4\tIN MX 10 $tunneloptions{mailhost}\n";
	   $s->{tunrdns}.="$rev4\tIN PTR $name.$tunneloptions{domain}.\n";
   }
   $s->{hosts}.="$ip4\t$name.$tunneloptions{domain}\t$name\n";

# IPv6
   my @eui64=("0000","0000","0000",sprintf("%04i",$ippart));
   my $ip6=$tunneloptions{prefix}.":".join(":", @eui64);
   if($flags=~m/6/) {
      $s->{fdns6}.="${name}\tIN AAAA $ip6\n";
      $s->{fdns6}.="${name}6\tIN AAAA $ip6\n";
   }
# even ipv4-only hosts can have reverse ipv6 mappings without causing trouble
   my $n6=ipcalc::ip6_to_rev(join("",@eui64));
   $s->{tunrdns6}.="$n6\tIN PTR $name.$tunneloptions{domain}.\n";
#print "hostlist.pm\n$s->{fdns6}";
}

});

# prefilterhooks check include mtime
push(@::prefilterhooks, sub{
      my($context, $infilename, $basefilename, $outfilename, $needpost, $needcheck)=@_;
      my $c=getFileContent($infilename);
      if($c=~m/##(rdns|rdns6|fdns|fdns6|dhcp|tunrdns)##/g) {
         my $touchfilename=$infilename.".$context.hostlist.touch";
         my $t=getmtime($touchfilename);
         if($t<getmtime($tunneloptions{infile})) {# || $t<getmtime("conf.d/hostlist.pm"))) {
            $_[5]=1; 
            diag("include $tunneloptions{infile} changed => rechecking $basefilename"); 
            touch($touchfilename);
         }
      }
   });

1;
