#!/usr/bin/perl -w
# by Bernhard M. Wiedemann <bernhard lsmod.de> 
# and Rene Kray <rene kray.info>
# licensed under GNU GPL
# $Id: configurer.pl,v 1.10 2011-06-13 13:24:12 cvs Exp $

use strict;
use File::Find;
use Getopt::Long;
use xbase;


# default config
our %options=qw(
force 0
forcepost 0
debug 1
copy 1
sshtimeout 30
out out
);

my @options=qw(force! forcepost! debug! copy! out=s sshtimeout=i);

# substitutions
our %s=qw(
);

our @prefilterhooks;
our @filefilterhooks;
our @filedonehooks;
our @contextstarthooks;
our @contextdonehooks;
our @donehooks;


sub diag
{ if($options{debug}) {print @_,"\n"}}


# pull in configs
my $touchmtime=getmtime($options{out});
for my $conf (<conf.d/*.pm>) {
	if($touchmtime<getmtime($conf)) {$options{force}=1; diag("$conf changed - forcing full update")}
	do $conf;
}
if($touchmtime<getmtime($0)) {$options{force}=1; diag("$0 changed - forcing full update")}

GetOptions(\%options, @options);
if($options{forcepost}) {$options{force}=1}


sub dohooks
{
	my $funclist=shift;
	foreach my $func (@$funclist) {
		&$func # call with remaining original parameters
	}
}

our $context;
our $substitutes;
our $didwrite;
sub substitute($)
{my ($in)=@_;
	$in=~s/##include ([^#]+)##/getFileContent($1)/ge;
	$in=~s/##(\w+)##/$substitutes->{$1} or warn "subst $1 not defined"/ge;
	return $in;
}

sub handletemplate()
{
	if((-f $_) && /^in\/(.*)\.in$/) {
		my $basefilename=$1;
		my $mtime=getmtime($_);
		my $touchfilename="$_.$context.touch";
		my $outfilename="$options{out}/$context/$basefilename";
      my $needcheck=$options{force};
      my $needpost=$options{forcepost};
      my $mustupdate=(!-e $outfilename);
      dohooks(\@prefilterhooks, $context, $_, $basefilename, $outfilename, $mustupdate, $needcheck);
		if($needpost || $needcheck || $mustupdate || $mtime >getmtime($touchfilename)) {
			my $newout=getFileContent($_);
			dohooks(\@filefilterhooks, $context, $_, $basefilename, $outfilename, $mustupdate, $newout);
			$newout=substitute($newout);
			diag "checking $_";
			touch($touchfilename);
			if($mustupdate || getFileContent($outfilename) ne $newout) {
				diag "writing $outfilename";
				xbase::mkbasedir($outfilename);
				setFileContent($outfilename, $newout);
				$didwrite=1;
				$needpost=1;
			}
			if($needpost) {
				dohooks(\@filedonehooks, $context, $_, $basefilename, $outfilename);
			}
		}
	}
}

# main

foreach $context (keys(%s)) {
	$substitutes=$s{$context};
	$didwrite=0;
	dohooks(\@contextstarthooks, $context);
	find({wanted=>\&handletemplate, no_chdir=>1}, "in");
	dohooks(\@contextdonehooks, $context, $didwrite);
}

touch $options{out};
dohooks(\@donehooks);

